﻿/*
    <Rs485 Dinamic Library>
    Copyright (C) <2013>  <Ernesto Gigliotti>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace Rs485Lib
{
    [StructLayout(LayoutKind.Sequential)]
    public struct Port
    {
        public int cport_nr;
        public int bdrate;
    };

    class Rs485
    {
        private Port port;

        // Dll Rs485
        [DllImport("librs485_windows_amd64.dll", EntryPoint = "rs485OpenPort")]
        public static extern int rs485OpenPort(ref Port pPort);
        [DllImport("librs485_windows_amd64.dll", EntryPoint = "rs485SendPacket")]
        public static extern int rs485SendPacket(ref Port pPort, byte[] buff, int size);
        [DllImport("librs485_windows_amd64.dll", EntryPoint = "rs485ReceiveWithTimeout")]
        public static extern int rs485ReceiveWithTimeout(ref Port pPort, byte[] bufferRx, int size, int timeout);
        [DllImport("librs485_windows_amd64.dll", EntryPoint = "rs485Receive")]
        public static extern int rs485Receive(ref Port pPort, byte[] bufferRx, int size);
        [DllImport("librs485_windows_amd64.dll", EntryPoint = "rs485ClosePort")]
        public static extern void rs485ClosePort(ref Port pPort);
        //__________

        public Rs485(int portNumber, int baud)
        {
            port.cport_nr = portNumber;
            port.bdrate = baud;
        }

        public bool openPort()
        {
            if (rs485OpenPort(ref port) == 1)
            {
                rs485ClosePort(ref port);
                return false;
            }
            return true;
        }

        public int sendPacket(byte[] packet)
        {
            return rs485SendPacket(ref port, packet, packet.Length);
        }

        public byte[] receivePacketWithTimeout(int timeout)
        {
               byte[] buffRx = new byte[4096];
               int rb = rs485ReceiveWithTimeout(ref port, buffRx,4096,5000);
               if (rb != 0)
               {
                   byte[] r = new byte[rb];
                   int i=0;
                   while (i<rb)
                   {
                       r[i] = buffRx[i];
                       i++;
                   }
                   return r;
               }

               return null;
        }

        public byte[] receivePacket()
        {
            byte[] buffRx = new byte[4096];
            int rb = rs485Receive(ref port, buffRx, 4096);
            if (rb != 0)
            {
                byte[] r = new byte[rb];
                int i = 0;
                while (i < rb)
                {
                    r[i] = buffRx[i];
                    i++;
                }
                return r;
            }

            return null;
        }

        public void closePort()
        {
            rs485ClosePort(ref port);
        }

    }
}
